# A1.2 R Basics Variables.R
# Insight: Structuring your data is easy in R.
# str(), c(), typeof(), class(), is. ... (), plot(), attr(), names()
# Every variable is actually a vector
rm(list = ls())  # Take out the Environment "trash"
cat("\014") # Clear Console, making error checking easier.
while (!is.null(dev.list()))  dev.off() # Clear old plots
StrikePrice = 100 # Vector of length 1, data type is double
StrikePrice # Console: [1] 100
# Display STRucture
str(StrikePrice) # Console: num 100, data type is double, value is 100 
# Assignment of StockPrice to Price, = and <- perform same assignment
ExercisePrice <- StrikePrice[1] 
# StockPrices will be a numerical vector of size 5 (following 2 lines)
StockPrices =      # Note math symbol MUST be on top line (not true in C++)
 c(90, 95, 100, 105, 110)    # New line but same equation
# Alternative for line continuation is leaving an open parenthesis
StockPrices = ( 
  c(9.0, 9.5, 10.0, 10.5, 11.0))
# A compact summary of the particular data structure displayed in the console
str(StockPrices) # Summary
typeof(StockPrices) # What data type
class(StockPrices) # What data class
# Historically, single floating point variables were stored with accuracy 
#  around 7 digits whereas double floating point variables were about twice 
#  the precision or 14 digits. Note every machine is different. 
#  Details found when run .Machine.
.Machine # More details than you probably want to know
# Is it a double (R does not have single floating point)
is.double(StockPrices) 
is.integer(StockPrices) # Not an integer
StockPrices = 100L # Force data type to be integer
typeof(StockPrices) # Note converted to integer
is.double(StockPrices)
is.integer(StockPrices)
# Other data types
isCall = TRUE # binary or logical, TRUE = Anything else, FALSE = 0
typeof(isCall) 
is.double(isCall)
is.integer(isCall)
is.logical(isCall)
CallType = as.integer(isCall) # Note can convert to integer
CallType
CallType = 100000
isCall <- as.logical(CallType)
isCall
CallType = as.integer(isCall)
CallType
# Simple plots
PlotTitle = "Option Payoffs"
typeof(PlotTitle) # Character
is.double(PlotTitle)
is.integer(PlotTitle)
is.logical(PlotTitle)
is.character(PlotTitle)
length(StockPrices) # Number of elements
attributes(StockPrices) # Additional metadata -- none here
StockPrices = c(90, 95, 100, 105, 110)   
Payoffs = StockPrices - StrikePrice      # Note Payoffs is a vector
# Following line does not work as expected, just the last Payoffs variable
if(isCall)CallPayoffs = max(0, Payoffs) 
isCall = FALSE
if(!isCall) CallPayoffs = -99 # Test if isCall is not true, CallPayoffs = -99
# Need to run through a loop
for(i in 1:5){ # One way to handle vectors
  CallPayoffs[i] = max(0, StockPrices[i] - StrikePrice) 
}
rm(CallPayoffs)
# Alternatively use pmax (parallel max)
CallPayoffs = pmax(0, Payoffs)
# Want vector from 50 to 100, incremented by 2. (50, 52, ..., 148, 150)
StockPrices = c(1:51)
structure(StockPrices) # A complete listing of the data structure
for(i in 1:51){
  StockPrices[i] = 50 + (i-1)*2
}
StockPrices
rm(StockPrices)
# Another way
StockPrices = 2*c(25:75)
StockPrices
rm(StockPrices)
?'seq' # A simpler way
StockPrices = seq(50, 150, 2)
StrikePrice = 100
is.numeric(StrikePrice)
for(i in 1:51){
  CallPayoffs[i] = max(0, StockPrices[i] - StrikePrice)
}
plot(StockPrices, CallPayoffs)
rm(CallPayoffs)
# or
CallPayoffs = pmax(0, StockPrices - StrikePrice)
plot(StockPrices, CallPayoffs, type='l') # Note type is a line
# Example of a list rather than a vector
OptionName = 'Index Option'
isCall = TRUE
UnderlyingPrice = 105
StrikePrice = 100
InterestRate = 0.05
Volatility = 0.30
TimeToMaturity = 1
UnderlyingYield = 0.05
# Could use line continuation (+), but unnecessary
BSMOVM <- list(OptionName, isCall, UnderlyingPrice, StrikePrice, InterestRate, 
+  Volatility, TimeToMaturity, UnderlyingYield) 
BSMOVM <- list(OptionName, isCall, UnderlyingPrice, StrikePrice, InterestRate, 
  Volatility, TimeToMaturity, UnderlyingYield)
is.list(BSMOVM)
is.list(OptionName) # Is not a list
attr(BSMOVM,"Option Name") <- "S&P 500 Index Option"
str(BSMOVM) # Summary
str(attributes(BSMOVM))
# Names
names(BSMOVM) <- c("Option Name", "Is Call?", "Underlying Price", 
  "Strike Price", "Interest Rate", "Volatility", "Time To Maturity", 
  "Underlying Yield")
str(BSMOVM) # Summary
str(attributes(BSMOVM)) # Now has two levels of attributes
BSMOVM
# Vector of zeros
Dividends = numeric(4) # Vector of zeros, dividends at each node
