# BA1.3 R Basics Vocabulary
# Insight: Though it will take some effort, the R language will become intuitive.
# sapply(), match(), head(), tail(), all.equal(), is.finite(), is.infinite(),
# math functions, data.frame()
# Goal is to review relevant R functions that apply to quantitative finance
# See Hadley Wickham, _Advanced R_
# Identifying environment variables
rm(list = ls())  # Take out the Environment "trash"
cat("\014") # Clear Console, making error checking easier.
while (!is.null(dev.list()))  dev.off() # Clear old plots
x = -99 # An R object, a numeric vector of size 1
class(x) # One way to determine variable type
x = y = z = 0 # Define variables in one assignment, note x, y, and z in Environment Tab
a = 1; b = 2; c = 3 # Define variables on one line, separate by semicolons (;)
ls() # Lists in the Console all Environment variables
# Assign these variables to character vector named list
list = ls() # Assigns list to variable "list", see list in Environment Tab
# Remove all environment variables
rm(list=ls())
# Help
? mean # Description of arithmetic mean, see documentation in the Help Tab
help(median) # Alternative help display
x = c(1:5) # Define vector x of size 5, increment by 1, integer type (default)
str(x) # Display structure of an object in Console
# Assignments
a = 10; b <- 11; 12 -> c # Three equivalent ways to do assignment
x <- list("UG","G","UG","UG","G") # UnderGraduate or Graduate
# Operators
match(x,"G") # 1 if matched, NA if not matched
x %in% "G" # Logical TRUE and FALSE if matched
y <- c(1:10) # Create vector of length 10, increment from 1 by 1
z <- y[3] # One way to reference an element of vector y, z = 3L
names(x) <- c("Bob","Betty","Joe","Jack","Jane")
str(x) # Display structure of x
y <- x[1] # Creates list: Bob: chr "UG"
y <- x$Bob # Creates character vector: "UG"
head(x,3) # First three elements of list x
tail(x,2) # Last two elements of list x
rm(list=ls()) # Removes everything
# Others operators to learn: 
# subset: returns vector meeting certain condition
x <- c(1:100)
y <- subset(x, x > 50)
# assign 
x <- list("UG","G","UG","UG","G") 
names(x) <- c("Bob","Betty","Joe","Jack","Jane")
UG # Error
assign(x$Bob,100) # Assigns a value (100) to a name (UG)
x$Bob
UG # 100 due to assignment
y = x$Bob # y = "UG"
y = UG    # y = 100
# Comparison
a = 5
b = 1
c = a*b
all.equal(a,c) # Logical TRUE 
all.equal(a,b) # If false, provides mean relative difference
all.equal(b+4,c)
identical(a,c) # Logical TRUE
identical(a,b) # Logical FALSE
if(a!=b)d=10 # a!=b is true, thus d=10
if(a==b)d=20 # a==b is false, thus d=10
if(a>b)d=30 # a>b is true, thus d=30: similarly <
if(b>=c)d=40 #b>=c is false, thus d=30: similarly >=
?"ifelse"
a = c(-10:10) # Create an integer sequence from -10 to 10, by 1 (default)
b = sqrt(a) # Attempt to take the square root of each element
c = ifelse(a >= 0, a, NA)
a = NA # a is now "not available"
d <- is.na(a) # Hence, d is true
?"complete.cases"
x = c(1:5,NA) # Last item is not available
z = complete.cases(x) # Last item of z is FALSE
a = 3/0 # Not error, rather infinite
x = is.finite(a) # Logical FALSE
y = is.infinite(a) # Logical TRUE
x = c(1:2,-Inf, +Inf, NA) # Last item is not available
a = is.finite(x) # -Inf, Inf, and NA are not finite 
b = is.infinite(x) # Only - Inf and Inf are infinite
rm(list=ls())
# Math
a = 10
b = 20
c = a * b
d = a + b
e = a - b
f = a/b
g = a^b
rm(list=ls())
# Vector math
a = c(1:3)
b = c(3:1)
c = a * b
d = a + b
e = a - b
f = a/b
g = a^b
h = a %*% b # Outer product matrix multiplication
i = a %/% b # Integer division
j = b %/% a
rm(list = ls())
a = c(-3:3)
b = abs(a) # Returns vector of absolute values of a
c = sign(a) # -1 for negative, 0 for 0, and 1 for positive
d = sin(a) # All trig functions available: cos, tan, acos, asin, atan, atan2
rm(list = ls())
a = c(-3.93555, -2.2222, -1.1111, 0, 1.1111, 2.2222, 3.93555)
b = ceiling(a) # Smallest integer, not less than x
c = floor(a) # Largest integer, not greater than x
d = round(a, digits = 3) # Rounds at third significant digit
e = trunc(a) # Truncates at decimal
f = signif(a, digits = 2)  # Significant digits
g = exp(a) # Exponential
h = log(a) # Natural log
i = log10(a) # Log in base 10 (common)
j = log2(a) # Log in base 2 (binary)
k = sqrt(a) # Square root of each element of a
l = max(a) # maximum of vector a
m = min(a) # minimum of vector a
n = prod(a) # Returns the product of all values of a
o = sum(a) # Returns the sum of all values of a
rm(list=ls())
a <- array(10:1, dim=c(2,5)) # Matrix
b <- array(1:10)
print(a)
print(b)
c = cumsum(a[1,])  # Cumulative sums of the elements of the argument
d = cumsum(b)
print(c)
print(d)
c = cummax(a[1,]) # Maximum of elements of the argument
d = cummax(b)
e = cummin(a[1,])
f = cummin(b)
g = cumprod(a[1,])
h = cumprod(b)
rm(list=ls())
?"diff"
a <- seq(1, 20, 2) # Sequence, increment by 2
b = diff(a, lag = 2) # Difference, two elements apart
c = diff(a, differences = 2) # Difference, one element apart, twice (difference of difference)
d = diff(a, lag = 2, differences = 2)
lagt = 2 # Test lag
NV = 10 # Length of vector
e = a[(1+lagt):NV] - a[1:(NV-lagt)] # Difference lagged by lagt
f = diff(a, lag = 2)  # Difference again, lagged by two. NOTE: f = e
rm(list=ls())
a <- array(10:1, dim=c(2,5))
?"pmax"
print(a)
b = pmax(a[1,])
c = pmax(a[,1])
d = range(a[1,])
e = range(a[,1])
f = mean(b)
g = median(b)
h = cor(a[1,],b[1:5]) # Correlation
i = sd(a) # Standard deviation
j = var(a) # Variance 5x5 matrix of covariances
?"var"
?"sd"
k = sd(b)
l = var(b)
rm(list=ls())
# Logical
a = 10; b = 20; c = 30
if(a == b)c = -30
if(a != b)c = -30
# Vectors and matrices
a = c(1:5) # Numerical vector of size 5, initiallized with 1, 2, 3, 4, 5
b = matrix(data = 0.0, nrow = 5, ncol = 4) # Matrix 5x5
for(i in 1:5){
  a[i] = i
  for(j in 1:4){
    b[i,j] = i + 2*j # i references row, j references column
  }
}
a; b
rm(list=ls())
# Lists and data.frames
StockPrice = 100.0; StrikePrice = 100.0; InterestRate = 5.0
Volatility = 30; Maturity = 1; Ticker = "AAPL"
# list(): Creates a generic vector
InputData <- list(StockPrice, StrikePrice, InterestRate, Volatility, Maturity, Ticker)
sapply(InputData, class) # Apply class function to every element of InputData
class(InputData) # Apply class to InputData matrix
InputData
# names(): Functions to assign names of R objects
names(InputData) <- c("StockPrice", "StrikePrice", "InterestRate", "Volatility", 
  "Maturity", "Ticker")
sapply(InputData, class)
InputData
# Alternatively
InputData <- list(StockPrice, StrikePrice, InterestRate, Volatility, Maturity, Ticker)
InputData = array(data = InputData, dim = c(1, 6), dimnames = list(NULL, 
   c("StockPrice", "StrikePrice", "InterestRate", "Volatility", 
     "Maturity", "Ticker")))
sapply(InputData, class)
# data.frame is a preferred way to collect variables, a fundamental data structure
InputData = data.frame(matrix(vector(), 1, 6, dimnames=list(c(),
  c("StockPrice", "StrikePrice", "InterestRate", "Volatility", "Maturity", 
    "Ticker"))), stringsAsFactors=F)
InputData$StockPrice <- StockPrice
InputData$StrikePrice <- StrikePrice
InputData$InterestRate <- InterestRate
InputData$Volatility <- Volatility
InputData$Maturity <- Maturity
InputData$Ticker <- Ticker
sapply(InputData, class)
InputData
