# A1.4 R Basics Functions.R
# Insight: R has numerous packages with useful functions and you can build your own.
# ?, library(), search(), detach(), is. ... (), rnorm(), rlnorm(), mean(), hist(),
# function(), attr(), beep()
# Example 1: Built in base functions
rm(list = ls())  # Take out the Environment "trash"
cat("\014") # Clear Console, making error checking easier.
while (!is.null(dev.list()))  dev.off() # Clear old plots
# library and require load and attach add-on packages
?"library"
?"require" # Note it pulls up the same Help page
library(base) # Not required
require(base) # Not required
?"base" # Briefly explains the package
# library(help = "base") # Provides listing of functions
?"is.R" # Help for an existing function in base
# Some libraries modify existing functions, such as lubridate
# First, we need to be sure the package has been installed in the past.
# The next line only needs to ever be run once, but you are never sure where your R program will end up.
if("lubridate" %in% rownames(installed.packages()) == FALSE)install.packages("lubridate")
library(lubridate)
# Note that "date" is masked, thus there is more than one date function. 
# R will use the first available, depending on when packages were added.
# The order of packages can be found with the search function
search()
# At times you will want to remove some packages or detach them.
detach("package:lubridate", unload=TRUE) # Unattaching package
# Note that lubridate is removed.
search()
# if
if(is.R()){ # Functions with "is." are logical functions
  x <- 5
} else {
  x <- 10 # <ALT> <-> produces <- 
}
# c() function which combines arguments
help("c")
NumberOfDraws = 1000
StockReturns = c(1:NumberOfDraws)
# Function in stats (statistical package)
if("stats" %in% rownames(installed.packages()) == FALSE)install.packages("stats")
require(stats) # Likely not required if already installed
# help("stats")
# library(help = "stats") # Provides listing of functions
# rnorm function creates random numbers, normal(mean, sd)
help("rnorm")
ExpectedStockReturn = 0.15 # Expected stock return
SDStockReturn = 0.3 # Standard deviation of stock returns
# Vector of normally distributed random variables
StockReturns = rnorm(StockReturns, mean = ExpectedStockReturn, sd = SDStockReturn)
AverageStockReturns = mean(StockReturns)
Error = AverageStockReturns - ExpectedStockReturn
help("lognormal") # Functions are case sensitive, this does not workk
help("Lognormal") 
StockPrice = 100
MaturityTime = 1
# Excessive use of parenthesis makes reading equations easier for all involved
Drift = log(StockPrice) + (ExpectedStockReturn - ((SDStockReturn^2)/2.0))*MaturityTime
Noise = SDStockReturn * sqrt(MaturityTime)
Epsilon = c(1:NumberOfDraws)
Epsilon = rnorm(Epsilon)
# Lognormally distributed vector of terminal stock prices
StockPrices = rlnorm(Epsilon, mean = Drift, sd = Noise)
AverageStockPrice = mean(StockPrices)
ExpectedStockPrice = StockPrice * exp(ExpectedStockReturn*MaturityTime)
# Monte Carlo simulations always have sample error
SampleError = AverageStockPrice - ExpectedStockPrice
# Quick histogram to check simulation
hist(StockPrices, main="Lognormal Distribution", xlab="Prices", ylab="Frequency", border="blue", 
  col="green", xlim=c(0, 500), las=1, breaks=20)
#
# Nasty feature of R
# WRAPPED LINES MUST END WITH APPROPRIATE SIGN, NOT AT BEGINNING (z2 is wrong below)
#
rm(list = ls())
cat("\014") # Clear Console, making error checking easier.
dev.off() # Clear any old Plots
X = 100
Y = 200
z1 = X + Y
z2 = X         # Due to the absence of semicolons in R (vs. C++)
  + Y          # WRONG!!!
z3 = X +       # RIGHT: Always terminate line with required symbol
  Y
# Function that computes N + 1 (over simple to illustrate functions)
AddOne = function(N) {
  N + 1  # Alternatively: return(N+1)
}
attr(AddOne, "comment") <- "AddOne simply adds one to a number"
attr(AddOne, "help") <- "AddOne expects one number and it will add 1 to that number"
attributes(AddOne)
attr(AddOne, "comment")
attr(AddOne, "srcref") # Print R source code: SouRCe REFerences
N = 10
M = AddOne(N) # Call function AddOne and pass N. Then return results to M.
AddOne # What is this function? See the Console below.
#
# Miscellaneous libraries
#
install.packages("beepr")
if("beepr" %in% rownames(installed.packages()) == FALSE)install.packages("beepr")
library(beepr)
beep(sound = 2, print('Finished')) # Helpful when running long program
# Illustration of other available sounds
beep(sound = 1, print('Finished')) # Helpful when running long program
beep(sound = 3, print('Finished')) # Helpful when running long program
beep(sound = 4, print('Finished')) # Helpful when running long program
beep(sound = 5, print('Finished')) # Helpful when running long program
beep(sound = 6, print('Finished')) # Helpful when running long program
beep(sound = 7, print('Finished')) # Helpful when running long program
beep(sound = 8, print('Finished')) # Helpful when running long program
beep(sound = 9, print('Finished')) # Helpful when running long program
beep(sound = 10, print('Finished')) # Helpful when running long program
beep(sound = 11, print('Finished')) # Helpful when running long program
beep() # Default of one
