# A1.5 R Basics File Management.R
# Insight: Managing files is easy in R. Care must be taken with formatting.
# Insight: Managing dates is tricky and you must be extremely careful
# read(), head(), tail(), write(), paste(), load(), length(),
# read.xlsx(), write.xlsx(), length(), ncol(), nrow(), as.date(), array(), 
# data.frame()
# Required files:
#  A1.5a Options.csv (Comma separated values file)
# There are many packages related to MS Excel -- chose openxlsx 
#   Others include WriteXLS, xlsx, and XLConnect
rm(list = ls())  # Take out the Environment "trash"
cat("\014") # Clear Console, making error checking easier.
while (!is.null(dev.list()))  dev.off() # Clear old plots
if("openxlsx" %in% rownames(installed.packages()) == FALSE)
  install.packages("openxlsx")
if("date" %in% rownames(installed.packages()) == FALSE)
  install.packages("date")
library(openxlsx)
library(date)
#
# Example 1: File provided. read.csv is part of the utils package 
#  within the base package
#
OptionExample <- read.csv("A1.5a Options.csv")
tail(OptionExample, 5) # Last five observations
OptionExample$X <- NULL # Remove column X
tail(OptionExample, 5) # Note column X is removed
# Removes observations prior to 8/30/2015 (ends 8/31/2015)
OptionExample<-OptionExample[!(OptionExample$date < 20150830),] 
# Write data to csv file
write.table(OptionExample, file = 'A1.5a Temp File.csv', quote = FALSE, 
  row.names = FALSE, sep = ",", col.names = TRUE)
OptionTable <- read.csv("A1.5a Temp File.csv")
tail(OptionTable, 5)
# Save as a text file
write.table(OptionExample, file = 'A1.5a Temp File 2.txt')
OptionText <- read.table("A1.5a Temp File 2.txt")
tail(OptionText, 5)
?"write.table"
rm(list = ls()) 
file.remove('A1.5a Temp File.csv', 'A1.5a Temp File 2.txt') # Delete files
#
# Example 2: Online files using quantmod
#
# quantmod - pulling stock data from internet (may or may not work)
#   as internet protocols constantly change
if("quantmod" %in% rownames(installed.packages()) == FALSE)
  install.packages("quantmod")
library(quantmod)
Security1 <- "AAPL" # Ticker
# Input S1Data from Yahoo (or other available vendor) with 
#  getSymbols (quantmod function)
# load historical data into an xts object 
S1Data <- getSymbols(Security1, auto.assign = FALSE) 
# xts denotes eXtensible Time-Series object useful for managing time series 
# data (details later in 2.9 Zoo and Natural Gas Futures.R)
S1Data[1:5,] # Alternative way to see first five observations
# Helpful to make xts data generic for further processing (need to eliminate 
# ticker symbol). Now referenced as Security 1 (S1)
# Convert xts to numeric, strips index and returns only observation
S1GenericData <- coredata(S1Data)
tail(S1GenericData, 5) # Note loss of date
# Must use S1GenericData so AdjClose is not overwritten
S1AdjPrices <- data.frame("Date" = index(S1Data), 
  "AdjClose" = S1GenericData[,6])
head(S1AdjPrices, 5)
class(S1AdjPrices$Date)
# Eliminate data before a certain date
StartDate <- as.Date('2010-01-01')
class(StartDate)
S1AdjPrices <- S1AdjPrices[S1AdjPrices$Date >= StartDate,]
write.table(S1AdjPrices, file = 'A1.5a Temp Stock Data.txt', quote = FALSE, 
  row.names = FALSE, sep = ",", col.names = TRUE)
S1AdjPrices1 <- read.table('A1.5a Temp Stock Data.txt', sep = ',', 
  header = TRUE)
getwd() # Get working directory just to be sure where to locate files
?"save"
# Save as a binary R data set
save(S1AdjPrices, file = 'A1.5b Stock 1 Adjusted Prices.Rdata')
remove(S1AdjPrices) # Note S1AdjPrices is removed from the Environment
rm(list = ls()) 
# Upload a binary R data set
# Note S1AdjPrices is in the Environment
load('A1.5b Stock 1 Adjusted Prices.Rdata')  
head(S1AdjPrices, 5)
rm(list = ls()) 
file.remove('A1.5b Stock 1 Adjusted Prices.Rdata') # Alternative way to delete
#
# Example 3: Files on your computer (txt)
#
# If file is in the project path (relative path)
Rel_S1 <- read.table('A1.5a Temp Stock Data.txt', sep = ',', header = TRUE)
Rel_S1[1:5,]
# If file is not in the project path (absolute path)
Abs_S1 <- read.table('/QFRepository/Ch Appendix/A1.5a Temp Stock Data.txt', 
  sep = ',', header = TRUE)
Abs_S1[1:5,]
TestData <- read.table('A1.5a Temp Stock Data.txt', sep = ",", header = TRUE, 
  skip = 10, nrows = 20, col.names = c("Date", "AdjClose"))
TestData # 20 rows of data, starting at observation 11
# If need to access a particular path several times
Path <- '/QFRepository/Ch Appendix/'
File <- 'A1.5a Temp Stock Data.txt'
FileLocation <- paste(Path, File, sep = "")
Abs_S1 <- read.table(FileLocation, sep = ',', header = TRUE)
Abs_S1[1:5,]
rm(list = ls())
# Relative path using delim for delimiter
Del_S1 <- read.delim('A1.5a Temp Stock Data.txt', sep = ',', header = TRUE)
Del_S1[1:5,]
# Relative path using table, note header = TRUE is the default
# Note using table introduces heading V1-V7 as well as numerical precision 
# What is displayed is what is actually stored
Tab_S1 <- read.table('A1.5a Temp Stock Data.txt', sep = ',')
Tab_S1[1:5,]
# Replace 2.5 Temp Stock Data.txt with space delimited file
write.table(Del_S1, file = 'A1.5b Temp Stock Data.txt', quote = FALSE, 
  row.names = FALSE, sep = " ", col.names = TRUE)
Space_S1 <- read.delim('A1.5b Temp Stock Data.txt', sep = ' ', header = TRUE)
head(Space_S1, 5)
rm(list = ls()) 
# Read in lines using readLines, just messing around
# Each line is a character vector
RL_S1 <- readLines('A1.5b Temp Stock Data.txt') 
head(RL_S1,5)
length(RL_S1)
# Transpose and split character vector
RL1_S1 <- t(sapply(strsplit(RL_S1[1], " "),unlist)) # Read first line
head(RL1_S1, 5)
length(RL1_S1)
RL2_S1 <- t(sapply(strsplit(RL_S1[2], " "),unlist)) # Read second line
head(RL2_S1, 5)
length(RL2_S1)
# Clean up files
file.remove('A1.5b Temp Stock Data.txt') 
rm(list = ls())  # Clean up to illustrate xlsx
#
# Excel file management examples
#
# Example 1: Read in an existing data file, 2.7 AAPL_Yahoo.xlsx
#
AAPL_From_Excel <- read.xlsx(xlsxFile = "A1.5a AAPL_Yahoo.xlsx", sheet = 1, 
  skipEmptyRows = FALSE)
is.data.frame(AAPL_From_Excel) # Inputted as data frame
# Note that date is in "Julian" format (large integer number)
head(AAPL_From_Excel, 5)
tail(AAPL_From_Excel, 5)
# Note: attach() is a useful function that makes the data frame local in scope
# Do not have to use AAPL_From_Excel$ on right hand side
attach(AAPL_From_Excel) 
AAPL_From_Excel$Date2 = date.mmddyyyy(Date) # Date adjustment
detach(AAPL_From_Excel) # Revert back
# Excel Julian dates start from 12/30/1899
AAPL_From_Excel$Date2 = date.mmddyyyy(AAPL_From_Excel$Date) # Date adjustment
# But R defaults to 1/1/1960, thus the dates are off by 21,916 days
head(AAPL_From_Excel, 5)
tail(AAPL_From_Excel, 5)
# Date adjustment
AAPL_From_Excel$Date3 = date.mmddyyyy(AAPL_From_Excel$Date - 21916) 
head(AAPL_From_Excel,5)
tail(AAPL_From_Excel,5)
x1 = as.numeric(AAPL_From_Excel$Date)
x2 = as.date(AAPL_From_Excel$Date2) # Now Julian
AAPL_From_Excel$Open <- NULL
AAPL_From_Excel$High <- NULL
AAPL_From_Excel$Low <- NULL
AAPL_From_Excel$Close <- NULL
AAPL_From_Excel$Volume <- NULL
head(AAPL_From_Excel,5)
tail(AAPL_From_Excel,5)
AAPL_From_Excel$Date2 = NULL
AAPL_From_Excel$Date = AAPL_From_Excel$Date3
AAPL_From_Excel$Date3 = NULL
head(AAPL_From_Excel,5)
tail(AAPL_From_Excel,5)
#
# Example 2: Write out an existing data, AAPL_From_Excel
#
write.xlsx(AAPL_From_Excel, "A1.5b AAPL_AdjPrices.xlsx")
AAPLAdjPrices <- read.xlsx(xlsxFile = "A1.5b AAPL_AdjPrices.xlsx", sheet = 1, 
  skipEmptyRows = FALSE)
tail(AAPLAdjPrices,5)
FLength1 = length(AAPLAdjPrices)
FLength2 = ncol(AAPLAdjPrices)
FSize = nrow(AAPLAdjPrices)
TDate = as.date('06/01/2015') # Note in Julian format
Date1 = as.date(AAPLAdjPrices$Date)
j = 0
# Count number of days after TDate
for (i in 1:FSize){
  if(Date1[i] > TDate){
    j = j + 1
  }
}
TotalDates = j
# ARRAY PROBLEMS: All variables would be character, thus dfReturns[,5] 
# is error. Avoid using array.
dfReturns = array(data = -99, dim = c(TotalDates, 7), dimnames = list(NULL,
  c("Date", "Month", "Day", "Year", "Price", "Return", "Ticker")))
class(dfReturns) # data type for dfReturns
sapply(dfReturns, class) # data type for each variable within dfReturns
Ticker = 'AAPL'
for (i in 1:TotalDates){
  if(Date1[i] > TDate){
    dfReturns[i,5] = as.numeric(AAPLAdjPrices$Adj.Close[i])
    if(i > 1)dfReturns[i,6] = (dfReturns[i,5]/dfReturns[i-1,5]) - 1
    # If next line is uncommented, the for loop throws an error
    #    dfReturns[i,7] = Ticker 
# Assigning character converts all variables to character!
  }
}
# Note that all variables are character; hence, division fails 
#  (when Ticker assigned)
head(dfReturns, 3) 
dfReturns[,7] <- Ticker
head(dfReturns, 3) 
# data.frame is a preferred way to collect variables, 
#  a fundamental data structure
dfReturns = data.frame(matrix(vector(), TotalDates, 7, dimnames=list(c(), 
  c("Date", "Month", "Day", "Year", "Price", "Return", "Ticker"))), 
  stringsAsFactors=F)
sapply(dfReturns, class)
Ticker = 'AAPL'
Price = c(1:TotalDates)
NDate = c(1:TotalDates)
for (i in 1:TotalDates){
  if(Date1[i] > TDate){
    temp = date.mdy(Date1[i])
    dfReturns[i,1] = Date1[i] + 21916
    dfReturns[i,2] = temp$month
    dfReturns[i,3] = temp$day
    dfReturns[i,4] = temp$year
    # dfReturns[i,5] = as.numeric(AAPLAdjPrices$Adj.Close[i])
    dfReturns[i,5] = AAPLAdjPrices$Adj.Close[i]
    if(i > 1)dfReturns[i,6] = (dfReturns[i,5]/dfReturns[i-1,5]) - 1
    dfReturns[i,7] = Ticker
  }
}
sapply(dfReturns, class) # Variable types preserved
head(dfReturns,5)
write.xlsx(dfReturns, "A1.5b AAPL_Returns.xlsx")
# Clean up
file.remove('A1.5b AAPL_AdjPrices.xlsx')
file.remove('A1.5b AAPL_Returns.xlsx')
#
# BatchGetSymbols method of online data extraction
#
if("BatchGetSymbols" %in% rownames(installed.packages()) == FALSE)
  install.packages("BatchGetSymbols")
library(BatchGetSymbols)
library(zoo)
library(date)
library(openxlsx)
# Single stock
TRADE1 <- BatchGetSymbols('AAPL', # Symbol desired
  # src = "google", # Source: Yahoo or Google
# '1950-01-01', # Sys.Date()-365, start date in Date format  
  first.date = '1986-06-01', 
  last.date = Sys.Date()) # End date in Date format
head(TRADE1, 5)
head(TRADE1$df.tickers$price.close, 5)
head(TRADE1$df.control$ticker, 5)
head(TRADE1$df.control, 5)
head(TRADE1$df.tickers, 5)
plot(TRADE1$df.tickers$ref.date, TRADE1$df.tickers$price.close)
#
# Portfolio
#
Tickers = c('SPY', 'XLB', 'XLE', 'XLF', 'XLI', 'XLK', 'XLP', 'XLRE',
  'XLU', 'XLV', 'XLY')
PORTFOLIO1 <- BatchGetSymbols(tickers = Tickers, # Symbol desired
  bench.ticker = "^GSPC", # Benchmark (SP 500 for dates ?)  
  #  src = "yahoo", # Source: Yahoo or Google
  first.date = as.Date('2010-01-01'), # Sys.Date()-365, start date in Date format
  last.date = Sys.Date(),
  thresh.bad.data = 0.9) # Percent of bad data allowed, if less, out
is.data.frame(PORTFOLIO1)
is.numeric(PORTFOLIO1)
print(PORTFOLIO1$df.control)
print(PORTFOLIO1$df.tickers)
head(PORTFOLIO1, 5)
head(PORTFOLIO1$df.tickers$price.close, 5)
head(PORTFOLIO1$df.control$ticker, 5)
Value <- PORTFOLIO1$df.tickers$price.adjusted[1]
head(Value, 5)
class(PORTFOLIO1)
str(PORTFOLIO1)
x1 <- PORTFOLIO1[[1]]
x2 <- as.data.frame(PORTFOLIO1[[2]])
x3 <- x2[x2$ticker == 'SPY',]
