# A1.6 R Basics Plot Example
# Insight: Illustrate various plots
# tidyverse, ggplot2, dplyr, readr, as.Date(), paste0(), head(), plot(), 
#  ggplot()
# tidyverse loads multiple packages with one command
# if("tidyverse" %in% rownames(installed.packages()) == FALSE)
#   install.packages("tidyverse")
# # if("scales" %in% rownames(installed.packages()) == FALSE)
# #  install.packages("scales")
# # if("readr" %in% rownames(installed.packages()) == FALSE)
# #  install.packages("readr")
# # if("dplyr" %in% rownames(installed.packages()) == FALSE)
# #  install.packages("dplyr")
# # if("ggplot2" %in% rownames(installed.packages()) == FALSE)
# #  install.packages("ggplot2")
# library(tidyverse) # shortcut for: library(ggplot2); library(dplyr); ...
# # library(scales)    # for date_format() function for ggplot2
# # library(readr)
# # library(dplyr)
# library(ggplot2)
# library(scales)
# library(quantmod) # reading data from internet
# Preferred method for adding multiple packages
# png package: Read and write PNG images
# grDevices package: Support for colors and fonts
PackagesToLibrary <- c("tidyverse", "ggplot2", "scales", "stats", "grDevices", 
  "quantmod","png","grDevices") # Libraries
if (length(setdiff(PackagesToLibrary, rownames(installed.packages()))) > 0) {
  install.packages(setdiff(PackagesToLibrary, rownames(installed.packages())))  
} # Make sure libraries are installed on this computer
# Load and attach libraries
lapply(PackagesToLibrary, library, character.only = TRUE) 
rm(PackagesToLibrary)
rm(list = ls())  # Take out the Environment "trash"
cat("\014") # Clear Console, making error checking easier.
while (!is.null(dev.list()))  dev.off() # Clear old plots
# Globally set fonts for graphs: Great for consistency in documents
par(family = 'Times New Roman') 
names = LETTERS[1:26] # Gives a sequence of the letters of the alphabet
beta1 = rnorm(26, 5, 2) # N(5,2), a vector of slopes (one for each letter)
beta0 = 10 ## A common intercept
# 26 plots are produced, output to screen and files (see current directory)
for(i in 1:26){ # Standard for loop, runs from 1 to 26 by 1
  x = rnorm(500, 105, 10) # N(105, 10)
  y = beta0 + beta1[i]*x + 15*rnorm(500) # Fitted OLS with noise
  mytitle = paste("Graph for Letter ", names[i])
  plot(x, y, main = mytitle) # Plot to screen
# Save plots to Plot_A.jpg  
  PlotName <- file.path(paste("Plot_", names[i], ".jpg", sep = ""))
  jpeg(file=PlotName) # Produce plots in files
  mytitle = paste("File Plot ", names[i]) # Plot to file
  plot(x, y, main = mytitle) # Plot to file
  dev.off() # Shuts down producing plots in files
}
# unlink('*.jpg') # Rather dangerous, deletes all files with extension jpg
# Safer way to delete files (one at a time, cannot accidentally delete files)
for(i in 1:26){ 
  PlotName <- file.path(paste("Plot_", names[i], ".jpg", sep = ""))
  file.remove(PlotName) # Alternative way to delete
}
#
# Fancier Plots
#
# Insight: Generic calculations and plots can be separated from main code
# Inputs
StartDate <- as.Date("01/01/2017", format="%m/%d/%Y")
EndDate <- as.Date("01/01/2018", format="%m/%d/%Y")  
Security1 = "SO" # Southern Company ticker
# Input S1Data from Yahoo (or other available vendor) with 
#  getSymbols (quantmod function)
# load historical data into an xts object 
TCKR <- getSymbols(Security1, auto.assign = FALSE) 
# TCKRDate <- TCKR
tail(TCKR, 5) # 
TCKRGenericData <- coredata(TCKR)
tail(TCKRGenericData, 5) # Note loss of date
# Must use S1GenericData so AdjClose is not overwritten
TCKRAdjPrices <- data.frame("Date" = index(TCKR), 
  "Close" = TCKRGenericData[,4], "AdjClose" = TCKRGenericData[,6])
head(TCKRAdjPrices, 5)
class(TCKRAdjPrices$Date)
# # Convert to date format
# JDate is number of days since 1970-01-01
TCKRAdjPrices$JDate <- as.integer(TCKRAdjPrices$Date)
# dplyr method to manipulate dates
TCKRAdjPrices = TCKRAdjPrices %>% filter(Date<=EndDate, Date>=StartDate) %>% 
  mutate(Day = as.integer(format(Date, "%d")), 
  Month = as.integer(format(Date, "%m")), 
  Year = as.integer(format(Date, "%Y") )) %>% arrange(Date)
# Plot
x = TCKRAdjPrices$Date
y = TCKRAdjPrices$Close
Title1 = "Price Only"
yTitle = "Price"
xTitle = "Date"
plot(x, y, xaxt="n", type="l", col="blue", pch = 1, cex = 0.5, main=Title1, 
     xlab=xTitle, ylab=yTitle, las=1)
xat = seq(min(x), max(x), by="month")
xat.format = format(xat, "%d%b%Y")
axis(1,               # side=1 is x-axis
     at=xat,             # dates for ticks
     labels=xat.format,  # formatted date for printing
     las=1,              # las=2 for vertical, las=1 for horizontal
     cex.axis=0.8)        # text size
# Plot based on ggplot2
ggplot(TCKRAdjPrices) + geom_line(aes(Date, Close), color="blue", size=0.5) + 
  ggtitle("Price Only") + scale_x_date(labels=date_format("%d%b%Y"),
                                       date_minor_breaks="1 month")
#
# More on plots
#
library(data.table)  
library(date)
EvaluationDate = 20150831 #20150831 # Pick closest one: 20150713 to 20150831
EvaluationDTM = 25 # Pick closest one
OptionType = 0   # 1 for call, 0 for put
StrikeRange = 60 # Range above and below current price

dfTCKR <- fread("A1.6a AAPL_Yahoo.txt",header=TRUE)
keep <- c("Date", "Close")
# Note: Different syntax from basic data.frame
dfTCKR <- dfTCKR[,keep, with = FALSE] 
# Create JDate and trim dataset
dfTCKR$JDate = as.numeric(as.Date(as.character(dfTCKR$Date),
  format = "%Y-%m-%d"))
JEvaluationDate <- as.numeric(as.Date(as.character(EvaluationDate),
  format = "%Y%m%d"))
TestDate <- sort(dfTCKR$JDate, decreasing = FALSE)
k = 0
LengthTCKR <- length(TestDate)
for(i in 1:LengthTCKR){               # Find closest date or before
  if(TestDate[i] <= JEvaluationDate){
    TestJDate <- TestDate[i]  
    k = k + 1
  }
}
dfTCKR <- dfTCKR[dfTCKR$JDate == TestJDate,]
# Read in option data, trim, adjust strike price
dfTCKR_Options <- fread("A1.6a AAPL_1M.csv")
keep <- c("date", "exdate", "cp_flag", "strike_price", "best_bid", 
  "best_offer", "volume", "open_interest")
# Note: Different syntax from basic data.frame
dfTCKR_Options <- dfTCKR_Options[,keep, with = FALSE] 
dfTCKR_Options$strike_price <- dfTCKR_Options$strike_price/1000.0
dfTCKR_Options[1:5,]
dfTCKR_Options$DTM <- NA
# Select calls or puts
if(OptionType == 1)dfTCKR_Options <- 
  dfTCKR_Options[dfTCKR_Options$cp_flag == 'C',]
if(OptionType != 1)dfTCKR_Options <- 
  dfTCKR_Options[dfTCKR_Options$cp_flag == 'P',]
# Compute days to maturity
dfTCKR_Options$DTM = (as.numeric(difftime(
  as.Date(as.character(dfTCKR_Options$exdate),format = "%Y%m%d"),
  as.Date(as.character(dfTCKR_Options$date),format = "%Y%m%d"), 
  units = "days")))
dfTCKR_Options$JDate = as.numeric(as.Date(as.character(dfTCKR_Options$date),
  format = "%Y%m%d"))
dfTCKR_Options <- dfTCKR_Options[dfTCKR_Options$JDate == TestJDate,]

TestDTM <- sort(dfTCKR_Options$DTM, decreasing = FALSE)
k = 0
LengthDTM <- length(TestDTM)
for(i in 1:LengthDTM){               # Find closest date or before
  if(TestDTM[i] <= EvaluationDTM){
    TestRDTM <- TestDTM[i]  
    k = k + 1
  }
}
dfTCKR_Options <- dfTCKR_Options[dfTCKR_Options$DTM == TestRDTM,]
dfTCKR_Options$MidPrice <- (dfTCKR_Options$best_bid + 
  dfTCKR_Options$best_offer)/2.0
dfTCKR_Options <- merge.data.frame(dfTCKR_Options, dfTCKR, by.x = "JDate", 
  by.y = "JDate")
LengthTCKR <- length(dfTCKR_Options$JDate)
for(i in 1:LengthTCKR){
  if(dfTCKR_Options$cp_flag[i] == "C"){
    dfTCKR_Options$IntrinsicValue[i] <- max(0, dfTCKR_Options$Close[i] - 
      dfTCKR_Options$strike_price[i])
    dfTCKR_Options$TimeValue[i] <- max(0, dfTCKR_Options$MidPrice[i] - 
      dfTCKR_Options$IntrinsicValue[i])
  }
  if(dfTCKR_Options$cp_flag[i] == "P"){
    dfTCKR_Options$IntrinsicValue[i] <- 
      max(0, dfTCKR_Options$strike_price[i] - dfTCKR_Options$Close[i])
    dfTCKR_Options$TimeValue[i] <- max(0, dfTCKR_Options$MidPrice[i] - 
      dfTCKR_Options$IntrinsicValue[i])
  }
}

dfTCKR_Options <- dfTCKR_Options[dfTCKR_Options$strike_price >= 
  dfTCKR_Options$Close*(1 - (StrikeRange/100)),]
dfTCKR_Options <- dfTCKR_Options[dfTCKR_Options$strike_price <= 
  dfTCKR_Options$Close*(1 + (StrikeRange/100)),]
# Plot footers
TS = paste0('S=', round(dfTCKR_Options$Close[1],2))
TT = paste0(',T=',dfTCKR_Options$DTM,' Days to Maturity')
sTitle = paste0(TS, TT)
# Option value plots wrt strike price
dfTCKR_Options <- dfTCKR_Options[order(dfTCKR_Options$strike_price),]
MaxValue = max(dfTCKR_Options$MidPrice, dfTCKR_Options$IntrinsicValue)
MinValue = min(dfTCKR_Options$MidPrice, dfTCKR_Options$IntrinsicValue)
ylim1 = c(1:2); ylim1[1] = MinValue; ylim1[2] = MaxValue
MaxValue = max(dfTCKR_Options$strike_price)
MinValue = min(dfTCKR_Options$strike_price)
xlim1 = c(1:2); xlim1[1] = MinValue; xlim1[2] = MaxValue
legtxt = c("Option Mid Price","Lower Bound")
if(OptionType == 1){
  mTitle = "Call Options"
} else {
  mTitle = "Put Options"
}
yTitle = "Value"
xTitle = "Strike Price"
lTitle = "Parameter"
plot(dfTCKR_Options$strike_price, dfTCKR_Options$MidPrice, type = "l", 
  main = mTitle, sub = sTitle, xlab = xTitle, ylab = yTitle, col = "blue", 
  xlim = xlim1, ylim = ylim1, pch = 1, cex = 0.5)
lines(dfTCKR_Options$strike_price, dfTCKR_Options$IntrinsicValue, type = "l", 
  col ="red", xlim = xlim1, ylim = ylim1, pch = 2, cex = 0.5)
if(OptionType == 1){
  legend("topright", legtxt, cex = 0.75, lwd = c(1, 1), lty = c(1, 1), 
    col = c("blue","red"), pch = c(NA,NA), bty = "n", title = lTitle)
} else {
  legend("topleft", legtxt, cex = 0.75, lwd = c(1, 1), lty = c(1, 1), 
    col = c("blue","red"), pch = c(NA,NA), bty = "n", title = lTitle)
}
# Option time value plot wrt strike price
dfTCKR_Options <- dfTCKR_Options[order(dfTCKR_Options$strike_price),]
MaxValue = max(dfTCKR_Options$TimeValue)
MinValue = min(dfTCKR_Options$TimeValue)
ylim1 = c(1:2); ylim1[1] = MinValue; ylim1[2] = MaxValue
MaxValue = max(dfTCKR_Options$strike_price)
MinValue = min(dfTCKR_Options$strike_price)
xlim1 = c(1:2); xlim1[1] = MinValue; xlim1[2] = MaxValue
if(OptionType == 1){
  mTitle = "Call Options"
} else {
  mTitle = "Put Options"
}
yTitle = "Time Value"
xTitle = "Strike Price"
plot(dfTCKR_Options$strike_price, dfTCKR_Options$TimeValue, type = "l", 
  main = mTitle, sub = sTitle, xlab = xTitle, ylab = yTitle, col = "blue", 
  xlim = xlim1, ylim = ylim1, pch = 1, cex = 0.5)
