# A1.7 R Basics Calendar Challenges.R
# Some challenges with managing dates
rm(list = ls())  # Take out the Environment "trash"
cat("\014") # Clear Console, making error checking easier.
while (!is.null(dev.list()))  dev.off() # Clear old plots
par(family = 'Times New Roman') # Globally set fonts for graphs
PackagesToLibrary <- c("date","tis") # Libraries
if (length(setdiff(PackagesToLibrary, rownames(installed.packages()))) > 0) {
  install.packages(setdiff(PackagesToLibrary, rownames(installed.packages())))  
} # Make sure libraries are installed on this computer
# Load and attach libraries
lapply(PackagesToLibrary,library,character.only = TRUE) 
rm(PackagesToLibrary)
# Some challenges with calendar dates
#
# From Julian date to many other date functions
TestMonth <- 1
TestDay <- 10
TestYear <- 2005
sapply(TestMonth, class) # TestMonth has the "numeric" attribute
# Julian date, Jan 1, 1960 baseline date
# Function mdy.date(), defined in date package converts M, D, Y to Julian
TestDate <- mdy.date(TestMonth, TestDay, TestYear) # 16,446
sapply(TestDate, class) # TestDate has the "character" attribute
# TestDate formatted as "Class 'date' int 16446
is.integer(TestDate) # TRUE
is.date(TestDate) # TRUE
BaseDate <- mdy.date(1, 1, 1960) # BaseDate = 0
BaseDate <- mdy.date(1, 1, 1970) # BaseDate = 3,653 (10 years since 1/1/60)
DaysSinceBaseDate <- TestDate - BaseDate # 12,793 (+ 3,653 = 16,446)
# as.Date, in this context, expecting only integer. Thus, changing origin
# has no effect (?)
TestDate2 <- as.Date(TestDate, origin = "1970-01-01") # 2005-01-10
sapply(TestDate2, class) # TestDate2 has the "Date" attribute
is.character(TestDate2) # FALSE
TestDate2Int <- as.integer(TestDate2) # 12,793
# TestDate2 is same as original input, even with new origin
# Converting to solely integer, changes the date (roughly ten years later)
TestDate3 <- as.Date(TestDate, origin = "1970-01-01") # 2005-01-10
# Converting to integer adds a day
TestDate31 <- as.integer(TestDate) # 16,446
TestDate311 <- as.Date(TestDate31, origin = "1970-01-01") # 2015-01-11 (?)
sapply(TestDate3, class) # TestDate3 has the "Date" attribute
# Alternative coercion of character data to Julian date
CharDate = "01-10-2005"
TestDate4 <- as.date(CharDate) # 16,446
sapply(TestDate4, class) # TestDate4 has the "character" attribute
# Here just for completeness
TestDate5 <- as.POSIXlt(as.Date(TestDate4)) 
sapply(TestDate5, class) # TestDate5 has many variables with different attributes
TestDate5$sec
TestDate5$min
TestDate5$hour
TestDate5$mday 
TestDate5$mon + 1
TestDate5$year + 1900
TestDate5$wday
TestDate5$yday + 1
# But lose a day if
TestDate
TestDate6 <- as.POSIXlt(TestDate) 
TestDate6$sec
TestDate6$min
TestDate6$hour # Why 18???
TestDate6$mday # Lost one day
TestDate6$mon + 1
TestDate6$year + 1900
TestDate6$wday
TestDate6$yday + 1
# Current time
TestDate7 <- as.POSIXlt(Sys.time(), "GMT")
TestDate7$sec # Seconds, 0-61 (leap second ???)
TestDate7$min # Minutes, 0-59
TestDate7$hour # Hours, 0-23
TestDate7$mday # Day of month, 1-31
TestDate7$mon # January = 0; months after the first of the year
TestDate7$year # Years since 1900
TestDate7$wday # Day of week, 0-6, starting on Sunday
TestDate7$yday # Day of year, 0-365
# A look at 2005-01-10
TestDate8 <- as.POSIXlt(as.Date(16446, origin = "1960-01-01")) 
TestDate8$sec
TestDate8$min
TestDate8$hour 
TestDate8$mday 
TestDate8$mon + 1
TestDate8$year + 1900
TestDate8$wday
TestDate8$yday + 1
#
# Example 1 (as.date): MLK day (Monday, is holiday)
#
IMonth <- 1
IDay <- 21
IYear <- 2019
TestDateJulian1 <- mdy.date(IMonth, IDay, IYear, nineteen=TRUE)
CDate1 <- '1/21/2019'
TestDateJulian2 <- as.date(CDate1, order = "mdy")
CDate2 <- '1-21-2019'
TestDateJulian3 <- as.date(CDate2, order = "mdy")
CDate3 <- '21Jan2019'
TestDateJulian4 <- as.date(CDate3, order = "dmy") # Has no effect (dmy)
CDate4 <- 'January 21 2019'
TestDateJulian5 <- as.date(CDate4, order = "mdy")
CDate5 <- '21 1 2019'
TestDateJulian6 <- as.date(CDate5, order = "dmy") # Has effect
CDate6 <- '1212019'
TestDateJulian7 <- as.date(CDate6, order = "mdy") # Do not lead month w 0
CDate7 <- '2019-01-21'
TestDateJulian8 <- as.date(CDate7, order = "ymd") 
#
# Date format helps:
#
# %y: 2-digit year (07)
# %Y: 4-digit year (2007)
# %b: abbreviated month (Jan)
# %B: Unabbreviated month (January)
# %m: month as number (00-12 (?))
# %a: abbreviated weekday (Mon)
# %A: Unabbreviated weekday (Monday)
# %d: day as a number (0-31 (?))
today <- Sys.Date()
is.date(today)
class(today)
is.Date(today)
Today1 <- format(today, format="%B %d %Y")
Today1
Today2 <- format(today, format="%m/%d/%Y")
Today2
