# 3.9 Optimization Test.R
# function, optim
# Solving for implied parameters
# rmarkdown::render("3.9 Optimization Test.R", "word_document")
rm(list = ls())  # Take out the Environment "trash"
cat("\014") # Clear Console, making error checking easier.
while (!is.null(dev.list()))  dev.off() # Clear old plots
par(family = 'Times New Roman') # Globally set fonts for graphs
# Libraries
#  stats: integrate function
Packages <- c("stats") 
if(length(setdiff(Packages, rownames(installed.packages()))) > 0) {
  install.packages(setdiff(Packages, rownames(installed.packages())))
} # Make sure libraries are installed on this computer
lapply(Packages, library, character.only=TRUE) # Load and attach libraries
rm(Packages)
# Example 1: Minimize joint function (say option value and implied volatility)
#   Simple function assumed here for illustration, useful in Merton model related to
#   credit risk.
OptionValue = 14.23
Volatility = 30
# Specify the variables over which we optimize as the first input (a vector)
# Specify other parameters as second, third, etc. inputs
Solver1 <- function(variables, OptionValue, Volatility){
  variable1 <- variables[1]
  variable2 <- variables[2]
  return((variable1 - OptionValue)^2 + (variable2 - Volatility)^2)
}
# Minimization procedure
# First argument in optim is the vector of initial values of the variable optimized over, 
#   assumed zero here
# Second argument is the function to be minimized
# Third argumnet is the gradient of the function being minimized (can be NULL)
# Remaining argument are the parameters to be passed to fn and gr
# The following minimizes Solver1. To check the optimization procedure: we expect 
#   the result to be simply [14.23, 30]
OptimizationResults = optim(par = c(0,0), fn = Solver1, gr = NULL, method = "BFGS", 
  OptionValue, Volatility)
OptimizationResults
# Output: par - best set of parameters found
#         value - value of the function (fn) with parameters, par
#         convergence - 0 if successful, several potential error codes
ImpliedOptionValue = OptimizationResults$par[1]
ImpliedVolatility = OptimizationResults$par[2]
OptimiumValue = OptimizationResults$convergence
ImpliedOptionValue; ImpliedVolatility; OptimiumValue

