# 2.9 Forecasting Volatility Test.R 
# zoo, read.xlsx, lag, 
# Various volatility forecasts based on historical prices
# rmarkdown::render("2.9 Forecasting Volatility Test.R", "word_document")
rm(list = ls())  # Take out the Environment "trash"
dev.off() # Clear any old Plots
cat("\014") # Clear Console, making error checking easier.
# Install required packages if not installed
# if("zoo" %in% rownames(installed.packages()) == FALSE)install.packages("zoo")
#if("openxlsx" %in% rownames(installed.packages())==FALSE)install.packages("openxlsx")
# Attach required packages
library(zoo)
library(openxlsx)
library(stats)
#
# Import large data file of futures contract variables
#
FileName <- 'SPX.xlsx'
FP <- read.xlsx(FileName, sheet = 1, startRow = 1, colNames = TRUE,
  rowNames = FALSE, detectDates = FALSE, skipEmptyRows = TRUE,
  rows = NULL, cols = NULL, check.names = FALSE, namedRegion = NULL)
#
# Focus just on spot and spot returns (percentage change, continuously compounded)
#
Price <- FP$CSpot
Return <- FP$PCSpot
#
# Illustrate lag operator with and without zoo (indexed totally ordered observations)
#
# Lag operator does not produce lagged results on R matrices!
# The 'lag' function should lag a given time series by the number of lags given.
LagPrice <- lag(Price, -1, na.pad = TRUE) # -1 stands for first lag of Price
# First five entries of Price and LagPrice in Console
head(Price, 5)
head(LagPrice, 5) 
# Both vectors are equivalent. The failure of the lag operator in the above example 
# is due to the fact that the elements of a matrix are not subscripted with respect 
# to a time variable. We need to convert a matrix into time series, that is, 
# observations indexed by time. The zoo package provides this functionality.
ZooPrice <- zoo(Price)
LagPrice <- lag(ZooPrice, -1, na.pad = TRUE)
# Print first five entries of Price and LagPrice
head(Price, 5)  # Note that there is no index
head(ZooPrice,5) # Now we have an index
head(LagPrice, 5) # and hence LagPrice now has the expected behavior
# We can now construct returns from the price data
# Zoo structure is preserved when new variable created
MyReturn <- log(ZooPrice) - log(LagPrice) 
# Compare to the Return series from the data source
head(Return, 5)
head(MyReturn, 5)
#
# Forecasting volatility
#
# Volatility is defined as the standard deviation of continuously compounded returns.
# Use a small sample of the data (last 100 days)
Return <- zoo(tail(Return, 100))
# We begin with a window of 50 days (1st through 50th observation in our sample)
# The simplest estimator of volatility is the sample standard deviation.
# The loop below does following: Find standard devation of the first 50 observations, 
# the forecast of volatility on day 51. Next find standard deviation of the first
# 51 obsevations, the forecast of volatility on day 52. Continues until
# the forecast of volatility on day 100 is obtained.
VolatilityForecast <- zoo(numeric(50))
for(i in 1:50) VolatilityForecast[i] <- sqrt(var(head(Return, 49+i)))
VolatilityForecast
# Average volatility throughout the time period 
Estimator1 <- mean(VolatilityForecast)
Estimator1
# Compare the forecast to realized volatility 
RealizedVolatility <- sqrt(var(tail(Return, 50)))
RealizedVolatility 
# Note this data has realized volatity with an upward trend.
# Alternatively, consider using an exponentially weighted average
Weight <- exp(seq(1:50)/50) # create exponential weights
Weight <- Weight/sum(Weight) # scale the weights so they sum to one
# Note the vector multiplication, %*%, next
Estimator2 <- VolatilityForecast%*%Weight 
Estimator2 # This estimator is closer to the realized volatility
RealizedVolatility
